<?php
if (!defined('APS_DEVELOPMENT_MODE')) define('APS_DEVELOPMENT_MODE', 'on');

require_once "aps/2/runtime.php";
require 'contexts.php';

// Definition of type structures

class CPU
{
    /**
     * @type("integer")
     * @title("Number of CPUs")
     * @description("Number of CPU cores")
     */
    public $number;
}

class OS
{
    /**
     * @type("string")
     * @title("OS Name")
     * @description("Operating System Name")
     */
    public $name;

    /**
     * @type("string")
     * @title("OS Version")
     * @description("Operating System version")
     */
    public $version;
}

class Hardware
{
    /**
     * @type("integer")
     * @title("RAM Size")
     * @description("RAM size in GB")
     */
    public $memory;

    /**
     * @type("integer")
     * @title("Disk Space")
     * @description("Disk space in GB")
     */
    public $diskspace;

    /**
     * @type("boolean")
     * @title("VM with separate OS core or Container using shared OS core")
     * @description("VM requires more time for provisioning - async POST")
     */
    public $VM;

    /**
     * @type("CPU")
     * @title("CPU")
     * @description("Server CPU parameters")
     */
    public $CPU;
}

class Platform
{
    /**
     * @type("string")
     * @title("Architecture")
     * @description("Platform architecture")
     */
    public $arch;

    /**
     * @type("OS")
     * @title("OS Parameters")
     * @description("Parameters of operating system")
     */
    public $OS;
}


// Main class
/**
 * @type("http://aps-standard.org/samples/sample-basic/vps/1.0")
 * @implements("http://aps-standard.org/types/core/resource/1.0")
 */
class vps extends APS\ResourceBase
{

    // Relation with the management context
    /**
     * @link("http://aps-standard.org/samples/sample-basic/context/1.0")
     * @required
     */
    public $context;

    // VPS properties

    /**
     * @type("string")
     * @title("name")
     * @description("Server Name")
     */
    public $name;

    /**
     * @type("string")
     * @title("Description")
     * @description("Server Description")
     */
    public $description;

    /**
     * @type("string")
     * @title("state")
     * @description("Server State")
     */
    public $state;

    // VPS complex properties (structures) - defined as classes above

    /**
     * @type("Hardware")
     * @title("Hardware")
     * @description("Server Hardware")
     */
    public $hardware;

    /**
     * @type("Platform")
     * @title("Platform")
     * @description("OS Platform")
     */
    public $platform;

    /**
     * @type("integer")
     * @title("Counter for async operation")
     * @description("Counts down number of retries for async operation")
     */
    public $retry;

    // Add Notification ID as a VPS property
    /**
     * @type(string)
     * @title("Notification ID")
     */
    public $notificationId;

    // Standard CRUD operations

    // Provision method is called when a sync provisioning is requested
    public function provision()
    {

        // Get instance of the Notification Manager:
        $notificationManager = \APS\NotificationManager::getInstance();

        // Get ready account APS ID needed for the notification structure.
        //	For this purpose, use the VPS-to-context relation and the context-to-account relation in a chain.
        $accountId = $this->context->account->aps->id;

        // Create Notification structure
        $notification = new \APS\Notification;
        $notification->message = new \APS\NotificationMessage("VPS Creation");
        $notification->details = new \APS\NotificationMessage("VPS __name__ is provisioned", array("name" => $this->name));
        $notification->accountId = $accountId;
        $notification->status = \APS\Notification::ACTIVITY_READY;
        $notification->packageId = $this->aps->package->id;
        $notification->link = new \APS\NotificationLink("/v/aps/samples/sample-basic/servers/", "/v/aps/samples/sample-basic/server-edit/r/" . $this->aps->id, "View or edit VPS");

        // VM requires more time, thus the *async* operation is required
        if ($this->hardware->VM) {
            $this->state = "Creating"; // VPS state is intermediate

            // Edit the notification
            $notification->message->message = "VM Creation";
            $notification->details->message = "Provisioning of VM __name__ is started";
            $notification->status = \APS\Notification::ACTIVITY_IN_PROGRESS;
            $notification->link->linkMore = "/v/aps/samples/sample-basic/servers";
            $notification->link->linkMoreText->message = "View Servers";
        } else {
            // Simulate async provisioning
            $this->retry = 5;
        }

        $notificationResponse = $notificationManager->sendNotification($notification);

        // Store the Notification ID as the VPS property to update or remove it in other operations
        $this->notificationId = $notificationResponse->id;

        if ($this->hardware->VM) {
            //Require async operation:
            throw new \Rest\Accepted($this, "Creating VM", 5); // Returns to APSC code 202 Accepted
        }

        $this->context->resourceCounting();
    }

    // Async provisioning proposed by the provision() method and initiated by the system
    public function provisionAsync()
    {

        // Get instance of the Notification Manager:
        $notificationManager = \APS\NotificationManager::getInstance();

        // Get ready account APS ID needed for the notification structure. For this purpose,
        // use the VPS-to-context and context-to-account relations in a chain.
        $accountId = $this->context->account->aps->id;

        $this->retry += 1;                // Increment the retry counter

        if ($this->retry >= 5) {        // Loop until the 5th retry

            $this->context->resourceCounting();

            // Create Notification structure
            $notification = new \APS\Notification;
            $this->state = "Stopped";    // Finish the Async operation - return 200 OK to APSC
            $notification->message = new \APS\NotificationMessage("VM Creation");
            $notification->details = new \APS\NotificationMessage("Provisioning of VM __name__ is completed", array("name" => $this->name));
            $notification->status = \APS\Notification::ACTIVITY_READY;
            $notification->link->linkMore = "/v/aps/samples/sample-basic/server-edit/r/" . $this->aps->id;
            $notification->link->linkMoreText->message = "View or edit VM";

            // If Notification Manager is available, then send the notification
            if (isset($this->notificationId)) {
                $notificationResponce = $notificationManager->editNotification($this->notificationId, $notification);
            }

        } else {
            // Require the next async retry
            throw new \Rest\Accepted($this, "Creating VM", 5); // Return code 202 Accepted
        }
    }

    public function unprovision()
    {

        $this->context->resourceCounting();

        $accountId = $this->context->account->aps->id;

        // Get instance of the Notification Manager:
        $notificationManager = \APS\NotificationManager::getInstance();

        // Create Notification structure
        $notification = new \APS\Notification;
        $notification->message = new \APS\NotificationMessage("Server Removal");
        $notification->details = new \APS\NotificationMessage("Server __name__ is removed", array("name" => $this->name));
        $notification->type = \APS\Notification::TYPE_WARNING;
        $notification->accountId = $accountId;
        $notification->status = \APS\Notification::WARNING_UPCOMING;
        $notification->packageId = $this->aps->package->id;
        $notification->link = new \APS\NotificationLink("/v/aps/samples/sample-basic/servers", "/v/aps/samples/sample-basic/servers", "View Servers");

        // Get a list of VPSEs to count the remained ones
        $apsc = \APS\Request::getController();
        $vpses = $apsc->getResources('implementing(http://aps-standard.org/samples/sample-basic/vps/1.0)');

        // Removing the last server of the customer?
        if (count($vpses) <= 1) {
            $notification->details->message = "The last server __name__ is removed";
            $notification->status = \APS\Notification::WARNING_EXISTING;
        }

        // If Notification Manager is available, remove the notification
        if (isset($this->notificationId)) {
            // Remove the stored Notification
            $notificationManager->deleteNotification($this->notificationId);
            unset($this->notificationId);

            // Send new notification
            $notificationManager->sendNotification($notification);
        }
    }

    public function configure($new)
    {
        $this->context->resourceCounting();
        parent::configure($new);
    }

    // Custom functions called on pressing start/stop buttons in custom UI:
    /**
     * @verb(GET)
     * @path("/start")
     * @return(string,text/json)
     */
    public function start()
    {
        $this->context->resourceCounting();    // Accumulate resource usage

        $this->state = 'Running';
        $apsc = \APS\Request::getController();
        $apsc->updateResource($this);
    }

    /**
     * @verb(GET)
     * @path("/stop")
     * @return(string,text/json)
     */
    public function stop()
    {
        $this->context->resourceCounting();    // Accumulate resource usage

        $this->state = 'Stopped';
        $apsc = \APS\Request::getController();
        $apsc->updateResource($this);
    }
}
